/*FILE: structure_test.c

Programer: Sunethra Ramanan suna@physics.iitm.ac.in

Date: 6 Feb 2012

Version: Original

Revision-History:
	7 Feb 2012

Comments:
6 Feb 2012: This code demonstrates how to use a C structure.

7 Feb 2012: More stuff you can do with a structure.
*/


#include <stdio.h>
#include <stdlib.h>

/************* Function and Structures************************/
int
contents(void *struct_ptr);

double 
my_func(double);

double 
func2(double A, double x);

struct F
{
	double params;					/*parameters that the function needs*/
	
	/*pointer to the function*/
	double (*func)(double, double);
};

struct my_struct
{
	int i;								/*some integer*/
	
	double a;							/*some real no.*/
	
	double *array_ptr;		/*pointer to some array*/
	
	void *ptr;						/*some pointer*/
	
	double (*func)(double);	/*some function*/
	
	struct F my_F;				/*a variable of type struct F*/
	
	struct F *F_ptr;				/*pointer to a structure called F*/
};

/*******************************************************************/

int
main(void)
{
	int I;													/*some integer*/
	
	double A;												/*some real number*/
	
	/*declaring and initializing an array of 10 elements*/
	double array[10] = {1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.};
	
	double *a_ptr;									/*pointer pointing to double*/
	
	void *my_ptr;										/*some pointer*/
	void *struct_ptr;								/*pointer to the structure that can be passed to another function*/
	
	struct my_struct my_variable;		/*new data type of type my_struct*/
	
	struct F some_func;							/*a variable of type struct F*/
	struct F *params_ptr;						/*pointer of type struct*/
	
	/*intializations*/
	I = 10;
	
	A = 4.;
	
	my_ptr = &A;
	
	/*initializing a_ptr*/
	a_ptr = array;
	
	/*initializing some_func*/
	some_func.params = A;
	some_func.func = &func2;
	
	params_ptr = &some_func;
	
	/*initializing structure of type my_struct*/
	my_variable.i = I; /*integer*/
	my_variable.a = A; /*double*/
	my_variable.ptr = my_ptr; /*pointer*/
	my_variable.array_ptr = a_ptr; /*array*/
	my_variable.func = &my_func;   /*pointer to a function*/
	my_variable.my_F.params = A;   /*member of structure F*/
	my_variable.my_F.func = &func2; /*member of structure F*/
	my_variable.F_ptr = params_ptr; /*pointer to a structure*/
	
	/*initializing struct_ptr*/
	struct_ptr = &my_variable;
	
	/*calling another function to view contents of the structure*/
	contents(struct_ptr);
	
	return(0);
}

/********************** Defining functions *****************************/
double
my_func(double x)
{
	return(x);
}

double 
func2(double A, double x)
{
	return(A + x);
}

/******************** contents *****************************************/
int
contents(void *struct_ptr)
{
	int i;										/*some counter*/
	
	int I;											/*integer stored in the structure*/
	
	double A;										/*double stored in the structure*/
	
	double parameters;					/*parameters passed to the structure F*/
	
	double *Arrptr;							/*array pointer*/
		
	double (*f1)(double); 		/*pointer to function my_func*/
	
	void *my_ptr;								/*value of pointer stored in the structure*/
	
	struct F *f2_ptr;						/*pointer of type F*/
	
	struct F my_var_F;					/*a variable of type struct F*/
	
	/*accessing members of the structure*/
	I = ((struct my_struct *) struct_ptr) -> i;
	
	printf("\n Printing integer \n");
	printf("integer:%d \n", I);
	
	A = ((struct my_struct *) struct_ptr) -> a;
	
	printf("\n Printing double \n");
	printf("double:%f \n", A);
	
	my_ptr = ((struct my_struct *) struct_ptr) -> ptr;
	
	printf("\n Printing pointer which points to the address of A \n");
	printf("pointer:%p check:%p \n", my_ptr, &A);
	
	f1 = ((struct my_struct *) struct_ptr) -> func;
	
	printf("\n Printing value of function my_func passed as a pointer \n");
	printf("value of function evaluated at A: %f check:%f \n", (*f1)(A), my_func(A));
	
	/*elements of the array*/
	Arrptr = ((struct my_struct *) struct_ptr) -> array_ptr;
	
	printf("\n Printing array elements passed through the array_ptr \n");
	/*check*/
	for(i = 0; i < 10; i ++)
	{
		printf("Array element:%d, %f \n", i, Arrptr[i]);
	}
	
	/*accessing members of the structure F using F_ptr*/
	f2_ptr = ((struct my_struct *) struct_ptr) -> F_ptr;
	
	printf("\n Printing elements of the structure F using the pointer to the structure \n");
	/*check*/
	printf("parameter for the function func2 is: %f \n", f2_ptr -> params);
	printf("value of func2 at A is:%f check:%f  \n", (*f2_ptr -> func)(A,A), func2(A,A));
	
	/*accessing members of the structure F using a variable of type struct F*/
	my_var_F = ((struct my_struct *) struct_ptr) -> my_F;
	
	printf("\n Printing elements of the structure F using the struct itself \n");
	/*check*/
	printf("parameter for the function func2 is: %f \n", my_var_F.params);
	printf("value of func2 at A is:%f check:%f  \n", my_var_F.func(A,A), func2(A,A));
	
	return(0);
}

/********************************************************************************/
/********************************************************************************/
